import json

from django.http import HttpResponse
from django.views.decorators.cache import cache_control
from django.views.generic.detail import DetailView

from bulbs.content.views import BaseContentDetailView
from bulbs.poll.models import Poll
from bulbs.poll.serializers import PollPublicSerializer


class PollDetailView(BaseContentDetailView):
    model = Poll
    ordering_fields = "__all__"


class MergedPollDataView(DetailView):
    model = Poll

    def render_to_response(self, context, **response_kwargs):
        """
        This endpoint sets very permiscuous CORS headers.

        Access-Control-Allow-Origin is set to the request Origin. This allows
          a page from ANY domain to make a request to this endpoint.

        Access-Control-Allow-Credentials is set to true. This allows requesting
          poll data in our authenticated test/staff environments.

        This particular combination of headers means this endpoint is a potential
            CSRF target.

        This enpoint MUST NOT write data. And it MUST NOT return any sensitive data.
        """
        serializer = PollPublicSerializer(self.object)
        response = HttpResponse(
            json.dumps(serializer.data),
            content_type="application/json"
        )
        if "HTTP_ORIGIN" in self.request.META:
            response["Access-Control-Allow-Origin"] = self.request.META["HTTP_ORIGIN"]
            response["Access-Control-Allow-Credentials"] = 'true'

        return response

poll_detail = cache_control(max_age=600)(PollDetailView.as_view())
get_merged_poll_data = cache_control(max_age=600)(MergedPollDataView.as_view())



from mock import patch

import contextdecorator

class mock_vault(contextdecorator.ContextDecorator):
    """Decorator + context manager for mocking Vault secrets in unit tests.

    Usage:
            def test_vault(self):
                with mock_vault({'some/secret': {'my key': 'my value'}}):
                    self.assertEqual('my value', vault.read('some/secret'))

        .. OR ..

            @mock_vault({'some/secret': {'my key': 'my value'}})
            def test_vault(self):
                self.assertEqual('my value', vault.read('some/secret'))
    """

    def __init__(self, secrets=None):
        super(mock_vault, self).__init__()
        self.secrets = secrets or {}

    def __enter__(self):

        def read(path):
            if path not in self.secrets:
                raise Exception('Did not find secret key "{}" in mock vault: {}'.format(
                    path, self.secrets))
            return self.secrets[path]

        self.patched = patch('bulbs.utils.vault.read', side_effect=read)
        return self.patched.start()

    def __exit__(self, *args):
        self.patched.stop()
        return False



try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

import requests

from django.conf import settings


class RequestFailure(requests.exceptions.RequestException):
    """The request failed."""


class TunicClient(object):

    def get(self, path):
        headers = {
            'Authorization': 'Token {}'.format(settings.TUNIC_REQUEST_TOKEN),
        }
        if settings.TUNIC_STAFF_BACKEND_ROOT in path:
            url = path
        else:
            url = self.handle_protocol_relative_paths(path)
        resp = requests.get(url, headers=headers)
        if not resp.ok:
            raise RequestFailure(response=resp)
        return resp

    def handle_protocol_relative_paths(self, path):
        url_base = settings.TUNIC_STAFF_BACKEND_ROOT
        if url_base.startswith('//'):
            url_base = 'http:' + url_base
        return urljoin(
            url_base, settings.TUNIC_API_PATH.rstrip('/') + '/' + path.lstrip('/')
        )

    def get_campaigns(self, filter_active=False):
        campaigns = {}
        path = "/campaign/"
        if filter_active:
            path += "?active=True"
        while path:
            resp = self.get(path).json()
            campaigns.update({
                c["id"]: c for c in resp.get("results", {})
            })
            path = resp.get("next")
        return campaigns

    def get_active_campaigns(self):
        return self.get_campaigns(filter_active=True)



"""Logic for reading list behavior across all properties."""
from django.conf import settings
from django.core.cache import cache

from elasticsearch import TransportError
from elasticsearch_dsl import filter as es_filter

from bulbs.content.filters import NegateQueryFilter, SponsoredBoost
from bulbs.content.models import Content
from bulbs.content.search import randomize_es
from bulbs.sections.models import Section
from bulbs.special_coverage.models import SpecialCoverage
from .popular import get_popular_ids, popular_content
from .slicers import FirstSlotSlicer


class ReadingListMixin(object):
    """Mixin for Content-based objects to manage reading lists."""

    def _get_reading_list_identifier(self):

        # 1. Match content to sponsored Special Coverages
        results = self.percolate_special_coverage(sponsored_only=True)
        if results:
            return results[0]

        # 2."Popular" i.e., the content is one of the 25 most popular items.
        popular_ids = get_popular_ids()
        if popular_ids and self.id in popular_ids:
            return "popular"

        # 3. Any unsponsored special coverage reading list that contains this item.
        results = self.percolate_special_coverage()
        if results:
            return results[0]

        # 4. Any section that contains this item.
        try:
            results = Content.search_objects.client.percolate(
                index=self.mapping.index,
                doc_type=self.mapping.doc_type,
                id=self.id,
                body={"filter": es_filter.Prefix(_id="section").to_dict()}
            )
        except TransportError:
            results = {"total": 0}

        if results["total"] > 0:
            for result in results["matches"]:
                if not result["_id"].endswith("None"):
                    return result["_id"]

        return "recent"

    def validate_query(self, query):
        """Confirm query exists given common filters."""
        if query is None:
            return query
        query = self.update_reading_list(query)
        return query

    def get_validated_augment_query(self, augment_query=None):
        """
        Common rules for reading list augmentation hierarchy.

        1. Sponsored Content.
        2. Video Content.
        """
        augment_query = self.validate_query(augment_query)

        # Given an invalid query, reach for a Sponsored query.
        if not augment_query:
            augment_query = self.validate_query(Content.search_objects.sponsored())

        # Given an invalid Sponsored query, reach for a Video query.
        if not augment_query:
            reading_list_config = getattr(settings, "READING_LIST_CONFIG", {})
            excluded_channel_ids = reading_list_config.get("excluded_channel_ids", [])
            augment_query = self.validate_query(Content.search_objects.evergreen_video(
                excluded_channel_ids=excluded_channel_ids
            ))

        return augment_query

    def augment_reading_list(self, primary_query, augment_query=None, reverse_negate=False):
        """Apply injected logic for slicing reading lists with additional content."""
        primary_query = self.validate_query(primary_query)
        augment_query = self.get_validated_augment_query(augment_query=augment_query)

        try:
            # We use this for cases like recent where queries are vague.
            if reverse_negate:
                primary_query = primary_query.filter(NegateQueryFilter(augment_query))
            else:
                augment_query = augment_query.filter(NegateQueryFilter(primary_query))
            augment_query = randomize_es(augment_query)
            return FirstSlotSlicer(primary_query, augment_query)
        except TransportError:
            return primary_query

    def get_special_coverage_identifiers(self):
        cache_key = "special-coverage-identifiers-{}".format(self.id)
        identifiers = cache.get(cache_key)
        if identifiers is None:
            identifiers = self.percolate_special_coverage()
            cache.set(cache_key, identifiers, 60 * 5)
        return identifiers

    def get_reading_list_identifier(self):
        cache_key = "reading-list-identifier-{}".format(self.id)
        identifier = cache.get(cache_key)
        if identifier is None:
            identifier = self._get_reading_list_identifier()
            cache.set(cache_key, identifier, 60 * 5)
        return identifier

    def update_reading_list(self, reading_list):
        """Generic behaviors for reading lists before being rendered."""

        # remove the current piece of content from the query.
        reading_list = reading_list.filter(
            ~es_filter.Ids(values=[self.id])
        )

        # remove excluded document types from the query.
        reading_list_config = getattr(settings, "READING_LIST_CONFIG", {})
        excluded_doc_types = reading_list_config.get("excluded_doc_types", [])
        for obj in excluded_doc_types:
            reading_list = reading_list.filter(~es_filter.Type(value=obj))

        return reading_list

    def get_reading_list_context(self):
        """Returns the context dictionary for a given reading list."""
        reading_list = None
        context = {
            "name": "",
            "content": reading_list,
            "targeting": {},
            "videos": []
        }

        if self.reading_list_identifier == "popular":
            reading_list = popular_content()
            context.update({"name": self.reading_list_identifier})

            # Popular is augmented.
            reading_list = self.augment_reading_list(reading_list)
            context.update({"content": reading_list})
            return context

        if self.reading_list_identifier.startswith("specialcoverage"):
            special_coverage = SpecialCoverage.objects.get_by_identifier(
                self.reading_list_identifier
            )
            reading_list = special_coverage.get_content().query(
                SponsoredBoost(field_name="tunic_campaign_id")
            ).sort("_score", "-published")
            context["targeting"]["dfp_specialcoverage"] = special_coverage.slug
            if special_coverage.tunic_campaign_id:
                context["tunic_campaign_id"] = special_coverage.tunic_campaign_id
                context["targeting"].update({
                    "dfp_campaign_id": special_coverage.tunic_campaign_id
                })
                # We do not augment sponsored special coverage lists.
                reading_list = self.update_reading_list(reading_list)
            else:
                reading_list = self.augment_reading_list(reading_list)
            context.update({
                "name": special_coverage.name,
                "videos": special_coverage.videos,
                "content": reading_list
            })
            return context

        if self.reading_list_identifier.startswith("section"):
            section = Section.objects.get_by_identifier(self.reading_list_identifier)
            reading_list = section.get_content()
            reading_list = self.augment_reading_list(reading_list)
            context.update({
                "name": section.name,
                "content": reading_list
            })
            return context

        reading_list = Content.search_objects.search()
        reading_list = self.augment_reading_list(reading_list, reverse_negate=True)
        context.update({
            "name": "Recent News",
            "content": reading_list
        })
        return context

    def get_reading_list(self, published=True):
        """
        This is currently a misnomer, as it actually returns a dictionary object.
        The returned object contains the reading list.
        """
        return self.get_reading_list_context(published=True)

    @property
    def reading_list_identifier(self):
        _reading_list_identifier = getattr(self, "_reading_list_identifier", None)
        if not _reading_list_identifier:
            setattr(self, "_reading_list_identifier", self.get_reading_list_identifier())
        return self._reading_list_identifier



from djbetty.serializers import ImageFieldSerializer
from rest_framework import serializers

from .fields import RichTextField


class CopySerializer(serializers.Serializer):
    copy = RichTextField(required=True, field_size="long")


class ItemSerializer(CopySerializer):
    title = RichTextField(field_size="short")
    image = ImageFieldSerializer(required=False)


class XYItemSerializer(serializers.Serializer):
    title = RichTextField(field_size="short")
    copy_x = RichTextField(field_size="long")
    copy_y = RichTextField(field_size="long")


class ComparisonKeySerializer(serializers.Serializer):
    title = RichTextField(field_size="short")
    color = serializers.CharField()
    initial = serializers.CharField()


class ComparisonSerializer(serializers.Serializer):
    key_x = ComparisonKeySerializer()
    key_y = ComparisonKeySerializer()
    items = XYItemSerializer(many=True)


class ListInfographicDataSerializer(serializers.Serializer):
    is_numbered = serializers.BooleanField(default=False)
    items = ItemSerializer(many=True, required=False)


class ProConSerializer(serializers.Serializer):
    body = RichTextField(required=True, field_size="long")
    pro = CopySerializer(many=True)
    con = CopySerializer(many=True)


class StrongSideWeakSideSerializer(serializers.Serializer):
    body = RichTextField(required=True, field_size="long")
    strong = CopySerializer(many=True)
    weak = CopySerializer(many=True)


class TimelineSerializer(serializers.Serializer):
    items = ItemSerializer(many=True, required=False)



from collections import OrderedDict

from rest_framework import serializers
from rest_framework.metadata import SimpleMetadata
from rest_framework.utils.field_mapping import ClassLookupDict

from djbetty.serializers import ImageFieldSerializer

from .data_serializers import CopySerializer, ItemSerializer, XYItemSerializer
from .fields import ColorField, RichTextField
from .serializers import InfographicSerializer, InfographicDataField


def get_and_check_attribute(obj, attr_name):
    attribute = getattr(obj, attr_name, None)
    if not attribute:
        raise AttributeError("The provided object has no '{}' attribute.".format(attr_name))
    return attribute


class InfographicMetadata(SimpleMetadata):

    additional_attributes = ["field_size"]

    @property
    def label_lookup(self):
        mapping = SimpleMetadata.label_lookup.mapping
        mapping.update({
            ColorField: "color",
            CopySerializer: "array",
            ItemSerializer: "array",
            XYItemSerializer: "array",
            ImageFieldSerializer: "image",
            RichTextField: "richtext"
        })
        return ClassLookupDict(mapping)

    def determine_metadata(self, request, view):
        serializer_class = view.get_serializer_class()
        if issubclass(serializer_class, InfographicSerializer):
            data = self.get_custom_metadata(serializer_class, view)
            return data
        return super(InfographicMetadata, self).determine_metadata(request, view)

    def get_field_info(self, field):
        field_info = super(InfographicMetadata, self).get_field_info(field)
        for attr in self.additional_attributes:
            meta = getattr(field, attr, None)
            if meta:
                field_info.update({attr: meta})
        return field_info

    def get_serializer_info(self, serializer):
        serializer_info = super(InfographicMetadata, self).get_serializer_info(serializer)

        if hasattr(serializer, "child"):
            label = self.label_lookup[serializer.child]
        else:
            label = self.label_lookup[serializer]

        # MIKE PARENT PAY ATTENTION: IDK IF THIS IS PRUDENT. LET'S DISCUSS.
        if label != "field":
            return OrderedDict([
                ("type", label),
                ("fields", serializer_info)
            ])
        return serializer_info

    def get_custom_metadata(self, serializer, view):
        fields_metadata = dict()
        if hasattr(serializer, "__call__"):
            serializer_instance = serializer()
        else:
            serializer_instance = serializer
        for field_name, field in serializer_instance.get_fields().items():
            if isinstance(field, InfographicDataField):
                if view.suffix != "List":
                    serializer = view.get_object().get_data_serializer()
                    fields_metadata[field_name] = self.get_custom_metadata(serializer, view)
            elif isinstance(field, serializers.BaseSerializer):
                fields_metadata[field_name] = self.get_serializer_info(field)
            else:
                fields_metadata[field_name] = self.get_field_info(field)
        return {
            "fields": fields_metadata
        }



from django import template

register = template.Library()


@register.inclusion_tag('special_coverage/bulbs_sc_landing.html', takes_context=True)
def special_coverage_landing_partial(context, twitter_handle, share_message):
    context['twitter_handle'] = twitter_handle
    context['share_message'] = share_message
    return context



import jsonfield
from django_enumfield import enum
from elasticsearch_dsl import field

from bulbs.content.models import Content
from .enum import InfographicType
from .utils import get_data_serializer


class BaseInfographic(Content):

    infographic_type = enum.EnumField(InfographicType)
    data = jsonfield.JSONField()

    class Mapping(Content.Mapping):
        data = field.Object()

        class Meta:
            # Necessary to allow for our data field to store appropriately in Elasticsearch.
            # A potential alternative could be storing as a string., we should assess the value.
            dynamic = False

    def save(self, *args, **kwargs):
        self.full_clean()
        return super(BaseInfographic, self).save(*args, **kwargs)

    def clean(self, *args, **kwargs):
        self.validate_data_field()
        return super(BaseInfographic, self).clean(*args, **kwargs)

    def get_data_serializer(self):
        return get_data_serializer(self.infographic_type)

    def validate_data_field(self):
        Serializer = self.get_data_serializer()  # NOQA
        Serializer(data=self.data).is_valid(raise_exception=True)

    @classmethod
    def get_serializer_class(cls):
        from .serializers import InfographicSerializer
        return InfographicSerializer



class UncachedResponse(object):
    @property
    def default_response_headers(self):
        return {
            "Allow": ", ".join(self.allowed_methods),
            "Vary": "Accept",
            "Cache-Control": "no-cache"
        }



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('promotion', '0004_auto_20150403_1549'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pzoneoperation',
            name='polymorphic_ctype',
            field=models.ForeignKey(related_name='polymorphic_promotion.pzoneoperation_set+', editable=False, to='contenttypes.ContentType', null=True),
        ),
    ]



"""celery tasks for contributions."""
from celery import shared_task

from bulbs.content.models import Content
from .models import Contribution, FreelanceProfile


@shared_task(default_retry_delay=5)
def check_and_update_freelanceprofiles(content_id):
    content = Content.objects.get(id=content_id)
    for author in content.authors.all():
        profile = getattr(author, "freelanceprofile", None)
        if profile is None:
            FreelanceProfile.objects.create(contributor=author)


@shared_task(default_retry_delay=5)
def update_role_rates(contributor_role_pk):
    for contribution in Contribution.objects.filter(role__pk=contributor_role_pk):
        contribution.index()


@shared_task(default_retry_delay=5)
def run_contributor_email_report(**kwargs):
    from .email import EmailReport
    report = EmailReport(**kwargs)
    report.send_mass_contributor_emails()


@shared_task(default_retry_delay=5)
def check_and_run_send_byline_email(content_id, new_byline):
    from .email import send_byline_email
    content = Content.objects.get(id=content_id)
    removed_bylines = content.authors.exclude(pk__in=[c.id for c in new_byline])
    if removed_bylines:
        send_byline_email(content, removed_bylines)



from django.conf.urls import url, patterns

from bulbs.poll.views import get_merged_poll_data

urlpatterns = patterns(
    "",
    url(r"^poll/(?P<pk>[\d]+)/merged\.json$", get_merged_poll_data, name='get-merged-poll-data')
)



import json

from djes.models import Indexable

from django.conf import settings
from django.db import models
from django.template.defaultfilters import slugify

from djbetty.fields import ImageField

import requests

import six


class VideohubVideo(Indexable):
    """A reference to a video on the onion videohub."""
    title = models.CharField(max_length=512)
    description = models.TextField(blank=True, default="")
    keywords = models.TextField(blank=True, default="")
    image = ImageField(null=True, blank=True, alt_field="_image_alt",
                       caption_field="_image_caption")
    _image_alt = models.CharField(null=True, blank=True, editable=False, max_length=255)
    _image_caption = models.CharField(null=True, blank=True, editable=False, max_length=255)
    # External FK on videohub. Temp workaround until metadata refactor.
    channel_id = models.IntegerField(blank=True, null=True, default=None)

    # default values
    DEFAULT_VIDEOHUB_VIDEO_URL = "http://videohub.local/videos/{}"
    DEFAULT_VIDEOHUB_EMBED_URL = "http://videohub.local/embed?id={}"
    DEFAULT_VIDEOHUB_API_URL = "http://videohub.local/api/v0/videos/{}"
    DEFAULT_VIDEOHUB_API_SEARCH_URL = "http://videohub.local/api/v0/videos/search"

    class Mapping:
        class Meta:
            # Exclude image until we actually need it, to avoid dealing with custom mappings
            excludes = ('image',)

    @classmethod
    def get_serializer_class(cls):
        from .serializers import VideohubVideoSerializer
        return VideohubVideoSerializer

    @classmethod
    def search_videohub(cls, query, filters=None, status=None, sort=None, size=None, page=None):
        """searches the videohub given a query and applies given filters and other bits

        :see: https://github.com/theonion/videohub/blob/master/docs/search/post.md
        :see: https://github.com/theonion/videohub/blob/master/docs/search/get.md

        :param query: query terms to search by
        :type query: str
        :example query: "brooklyn hipsters"  # although, this is a little redundant...

        :param filters: video field value restrictions
        :type filters: dict
        :default filters: None
        :example filters: {"channel": "onion"} or {"series": "Today NOW"}

        :param status: limit the results to videos that are published, scheduled, draft
        :type status: str
        :default status: None
        :example status: "published" or "draft" or "scheduled"

        :param sort: video field related sorting
        :type sort: dict
        :default sort: None
        :example sort: {"title": "desc"} or {"description": "asc"}

        :param size: the page size (number of results)
        :type size: int
        :default size: None
        :example size": {"size": 20}

        :param page: the page number of the results
        :type page: int
        :default page: None
        :example page: {"page": 2}  # note, you should use `size` in conjunction with `page`

        :return: a dictionary of results and meta information
        :rtype: dict
        """
        # construct url
        url = getattr(settings, "VIDEOHUB_API_SEARCH_URL", cls.DEFAULT_VIDEOHUB_API_SEARCH_URL)
        # construct auth headers
        headers = {
            "Content-Type": "application/json",
            "Authorization": settings.VIDEOHUB_API_TOKEN,
        }
        # construct payload
        payload = {
            "query": query,
        }
        if filters:
            assert isinstance(filters, dict)
            payload["filters"] = filters
        if status:
            assert isinstance(status, six.string_types)
            payload.setdefault("filters", {})
            payload["filters"]["status"] = status
        if sort:
            assert isinstance(sort, dict)
            payload["sort"] = sort
        if size:
            assert isinstance(size, (six.string_types, int))
            payload["size"] = size
        if page:
            assert isinstance(page, (six.string_types, int))
            payload["page"] = page
        # send request
        res = requests.post(url, data=json.dumps(payload), headers=headers)
        # raise if not 200
        if res.status_code != 200:
            res.raise_for_status()
        # parse and return response
        return json.loads(res.content)

    def get_hub_url(self):
        """gets a canonical path to the detail page of the video on the hub

        :return: the path to the consumer ui detail page of the video
        :rtype: str
        """
        url = getattr(settings, "VIDEOHUB_VIDEO_URL", self.DEFAULT_VIDEOHUB_VIDEO_URL)

        # slugify needs ascii
        ascii_title = ""
        if isinstance(self.title, str):
            ascii_title = self.title
        elif six.PY2 and isinstance(self.title, six.text_type):
            # Legacy unicode conversion
            ascii_title = self.title.encode('ascii', 'replace')

        path = slugify("{}-{}".format(ascii_title, self.id))

        return url.format(path)

    def get_embed_url(self, targeting=None, recirc=None):
        """gets a canonical path to an embedded iframe of the video from the hub

        :return: the path to create an embedded iframe of the video
        :rtype: str
        """
        url = getattr(settings, "VIDEOHUB_EMBED_URL", self.DEFAULT_VIDEOHUB_EMBED_URL)
        url = url.format(self.id)
        if targeting is not None:
            for k, v in sorted(targeting.items()):
                url += '&{0}={1}'.format(k, v)
        if recirc is not None:
            url += '&recirc={0}'.format(recirc)
        return url

    def get_api_url(self):
        """gets a canonical path to the api detail url of the video on the hub

        :return: the path to the api detail of the video
        :rtype: str
        """
        url = getattr(settings, 'VIDEOHUB_API_URL', None)
        # Support alternate setting (used by most client projects)
        if not url:
            url = getattr(settings, 'VIDEOHUB_API_BASE_URL', None)
            if url:
                url = url.rstrip('/') + '/videos/{}'
        if not url:
            url = self.DEFAULT_VIDEOHUB_API_URL
        return url.format(self.id)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('special_coverage', '0006_auto_20151109_1708'),
    ]

    operations = [
        migrations.AddField(
            model_name='specialcoverage',
            name='end_date',
            field=models.DateTimeField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='specialcoverage',
            name='start_date',
            field=models.DateTimeField(null=True, blank=True),
        ),
    ]



from .enum import InfographicType
from .data_serializers import (
    ComparisonSerializer, ListInfographicDataSerializer, ProConSerializer,
    StrongSideWeakSideSerializer, TimelineSerializer
)


def get_data_serializer(infographic_type):
    serializer = {
        InfographicType.LIST: ListInfographicDataSerializer,
        InfographicType.TIMELINE: TimelineSerializer,
        InfographicType.STRONGSIDE_WEAKSIDE: StrongSideWeakSideSerializer,
        InfographicType.PRO_CON: ProConSerializer,
        InfographicType.COMPARISON: ComparisonSerializer
    }.get(infographic_type, None)
    if serializer is None:
        raise KeyError(
            """The requested Infographic does not have a configured serializer."""
        )
    return serializer



from rest_framework import serializers


class ColorField(serializers.CharField):
    pass


class RichTextField(serializers.CharField):

    def __init__(self, *args, **kwargs):
        self.field_size = kwargs.pop("field_size", None)
        super(RichTextField, self).__init__(*args, **kwargs)



from django.conf.urls import url, patterns
from django.views.decorators.cache import cache_control

from .views import GlanceFeedViewSet, RSSView, SpecialCoverageRSSView


urlpatterns = patterns(
    "",
    url(r"^rss", cache_control(max_age=300)(RSSView.as_view()),
        name="rss-feed"),
    url(r"^sc-rss", cache_control(max_age=300)(SpecialCoverageRSSView.as_view()),
        name="sc-rss-feed"),
    url(r"^glance.json$", cache_control(max_age=300)(GlanceFeedViewSet.as_view({'get': 'list'})),
        name="glance-feed"),
)



from .parser import parse_body


def transform(html, renderer):
    """Convert blob of body content HTML into another output format (ex: FB Instant Article).
    This wires up separate "parser" and "renderer" subsystems.
    """
    return renderer.generate_body(parse_body(html))
